#!/usr/bin/env perl

use strict;
use warnings;

use File::Copy;
use File::Basename;
use Cwd qw/getcwd/;

my $author = "zv";
my $fail_test = "";
my $top_dir = getcwd;
my $problem = basename($top_dir);
my $sol_name = $problem . "_" . $author;

sub compile_cpp {
    my $name = shift;
    system("g++ $name.cpp -O3 -o $name") == 0 or die "Cannot compile $name";
}

sub compile_java {
    my $name = shift;
    system("javac $name.java") == 0 or die "Cannot compile $name";
}

sub run_java {
    my $name = shift;
    system("java -ea $name") == 0 or die "Cannot run $name";
}

sub run_cpp {
    my $name = shift;
    system("./$name") == 0 or die "Cannot run $name";
}

`./clean.sh`;

mkdir "tests";
chdir "src";
print "Compiling generator\n";
compile_java("TestsGen");
print "Generating tests\n";
run_java("TestsGen");
chdir "..";

print "Compiling validator\n";
compile_java("Validate");

print "Compiling author solution\n";
compile_cpp($sol_name) if (-e ($sol_name . ".cpp"));
compile_java($sol_name) if (-e ($sol_name . ".java"));

# Only support java checkers
system("javac -cp ../../../lib/testlib4j.jar Check.java -encoding utf8") == 0 or die "Cannot compile checker";

sub generate_answers {
    my $test_dir = shift;
    print "Generation answers\n";
    for my $test (glob "./$test_dir/*") {
        next if $test =~ /\.a$/;
        print "Test $test\n";
        copy "$test", "$problem.in";
        system("java Validate < $problem.in") == 0 or die "Validator doesn't accept test $test";
        print "Validator accepts $test\n";

        run_cpp($sol_name) if (-e ($sol_name . ".cpp"));
        run_java($sol_name) if (-e ($sol_name . ".java"));
        copy "$problem.out", "$test.a";
        print "$test.a\n";
        system("java -Xmx256M -Xss64M -cp '../../../lib/testlib4j.jar:.' ru.ifmo.testlib.CheckerFramework Check $test $test.a $test.a");
    }
}

generate_answers "preliminary";
generate_answers "tests";

