#include <iostream>
#include <cstdio>
#include <vector>
#include <queue>

const int maxn = 1 << 17;
const int inf = 1e9 + 7;

using namespace std;

struct Graph {
    struct Edge {
        int from, to, cap, flow;
        Edge() {}
        Edge(int from, int to, int cap) : from(from), to(to), cap(cap), flow(0) {}
    };
    
    int n;
    int S, T;
    vector<Edge> edges;
    vector<vector<int> > e;
    vector<int> d, c;
    
    Graph() {}
    Graph(int n) {
        this->n = n;
        S = 0;
        T = n - 1;
        e.resize(n);
    }
    
    void addEdge(int from, int to, int cap) {
        e[from].push_back(edges.size());
        edges.push_back(Edge(from, to, cap));
        e[to].push_back(edges.size());
        edges.push_back(Edge(to, from, 0));
    }
    
    bool bfs() {
        d.assign(n, inf);
        c.assign(n, 0);
        queue<int> q;
        q.push(S);
        d[S] = 0;
        while (!q.empty()) {
            int v = q.front();
            q.pop();
            for (int i = 0; i < (int)e[v].size(); i++) {
                Edge cur = edges[e[v][i]];
                if (d[cur.to] > d[v] + 1 && cur.flow < cur.cap) {
                    d[cur.to] = d[v] + 1;
                    q.push(cur.to);
                }
            }
        }
        return d[T] != inf;
    }
    
    int dfs(int v, int flow = inf) {
        if (!flow) return 0;
        if (v == T) return flow;
        for (int &i = c[v]; i < (int)e[v].size(); i++) {
            Edge cur = edges[e[v][i]];
            if (d[cur.to] != d[v] + 1) continue;
            int o = dfs(cur.to, min(flow, cur.cap - cur.flow));
            if (o) {
                edges[e[v][i]].flow += o;
                edges[e[v][i] ^ 1].flow -= o;
                return o;
            }
        }
        return 0;
    }
    
    int getFlow() {
        int ans = 0;
        while (bfs()) {
            while (int pushed = dfs(S)) {
                ans += pushed;
            }
        }
        return ans;
    }
};

int n, m, k;
vector<pair<int, int> > e;
vector<int> a;

bool ok(int x) {
    Graph gr(n * x + 1);
    for (int i = 0; i < n; i++) {
        for (int j = 0; j < x - 1; j++) {
            gr.addEdge(i * x + j, i * x + j + 1, inf);
        }
    }
    for (int i = 0; i < (int)e.size(); i++) {
        for (int j = 0; j < x - 1; j++) {
            gr.addEdge(e[i].first * x + j, e[i].second * x + j + 1, 1);
            gr.addEdge(e[i].second * x + j, e[i].first * x + j + 1, 1);
        }
    }
    for (int i = 0; i < k; i++) {
        gr.addEdge((a[i] + 1) * x - 1, n * x, 1);
    }
    return gr.getFlow() == k;
}

int main() {
    freopen("messages.in", "r", stdin);
    freopen("messages.out", "w", stdout);

    cin >> n >> m;
    for (int i = 0; i < m; i++) {
        int u, v;
        cin >> u >> v;
        e.push_back(make_pair(--u, --v));
    }
    cin >> k;
    a.resize(k);
    for (int i = 0; i < k; i++) {
        cin >> a[i];
        a[i]--;
    }

    int l = 0, r = 2 * n + 1;
    while (r - l > 1) {
        int m = (l + r) >> 1;
        if (ok(m)) r = m;
        else l = m;
    }
    cout << r - 1 << endl;
    
    return 0;
}