import java.io.*;
import java.util.*;

import ru.ifmo.testlib.*;
import static ru.ifmo.testlib.Outcome.Type.*;

public class Check implements Checker {
    public Outcome outcome(Outcome.Type type, final String format, final Object... args) {
        return new Outcome(type, String.format(format, args));
    }

    class Point {
        int x, y;

        Point(int x, int y) {
            this.x = x;
            this.y = y;
        }
    };

    int n;
    Point[] p;
    int[] l, r;

    int sign(long x) {
        if (x < 0)
            return -1;
        if (x == 0)
            return 0;
        return 1;
    }

    boolean intersect(int i, int j, int ii, int jj) {
        long A1 = p[j].y - p[i].y;
        long B1 = p[i].x - p[j].x;
        long C1 = -A1 * p[i].x - B1 * p[i].y;

        long A2 = p[jj].y - p[ii].y;
        long B2 = p[ii].x - p[jj].x;
        long C2 = -A2 * p[ii].x - B2 * p[ii].y;

        return sign(p[i].x * A2 + p[i].y * B2 + C2) != sign(p[j].x * A2 + p[j].y * B2 + C2) &&
               sign(p[ii].x * A1 + p[ii].y * B1 + C1) != sign(p[jj].x * A1 + p[jj].y * B1 + C1);
    }

    int check(String answer, int[] ids) {
        if (!answer.equals("YES"))
            return -1;
        int[] deleted = new int[n];
        for (int i = 0; i < n; i++)
            deleted[i] = -1;
        for (int i = 0; i < n; i++) {
            if (ids[i] < 1 || ids[i] > 2) {
                return 0;
            }
            if (ids[i] == 1)
                ids[i] = l[i];
            else
                ids[i] = r[i];
            if (deleted[i] != -1 && ids[i] != deleted[i]) {
                return 0;
            }
            deleted[ids[i]] = i;
        } 
        int[] id = new int[n];
        for (int i = 0; i < n; i++) {
            if (deleted[i] == l[i])
                id[i] = r[i];
            else
                id[i] = l[i];
        }

        for (int i = 0; i < n; i++)
        for (int j = i + 1; j < n; j++)
            if (id[i] != j && intersect(i, id[i], j, id[j])) {
                return 0;
            }
        return 1;
    }

    @Override
    public Outcome test(InStream inf, InStream ouf, InStream ans) {
        n = inf.nextInt();
        p = new Point[n];
        for (int i = 0; i < n; i++) {
            int x = inf.nextInt();
            int y = inf.nextInt();
            p[i] = new Point(x, y);
        }
        l = new int[n];
        r = new int[n];
        for (int i = 0; i < n; i++) {
            l[i] = inf.nextInt();
            r[i] = inf.nextInt();
            l[i]--;
            r[i]--;
        }

        String pAnswer = ouf.nextToken();
        int[] pids = new int[n];
        if (pAnswer.equals("YES")) {
            for (int i = 0; i < n; i++) {
                pids[i] = ouf.nextInt();
            }
        }
        if (!ouf.seekEoF()) {
            return outcome(WA, "Too much tokens in answer");
        }

        String jAnswer = ans.nextToken();
        int[] jids = new int[n];
        if (jAnswer.equals("YES")) {
            for (int i = 0; i < n; i++) {
                jids[i] = ans.nextInt();
            }
        }
        if (!ans.seekEoF()) {
            return outcome(FAIL, "Too much tokens in jury answer");
        }

        int pCheck = check(pAnswer, pids);
        int jCheck = check(jAnswer, jids);

        if (jCheck == 0) {
            return outcome(FAIL, "Jury has wrong answer");
        }
        if (jCheck == -1 && pCheck == 1) {
            return outcome(FAIL, "Participant found answer, but jury didn't");
        }
        if (jCheck == 1 && pCheck == -1) {
            return outcome(WA, "Jury found answer, but participant didn't");
        }
        if (pCheck == 0) {
            return outcome(WA, "Participant has wrong answer");
        }

        return outcome(OK, "Correct answer");
    }
}

